/*************************************************
*  Program Name:  robot1.h
*
*  Functionality:  Provide setup information
*                  for the Unimate Puma 560
*                  robot.
*  
*  Author:   Jat S. Pannu
*  Modified: Jonathan Larson   on Jul  9, 1996
*  Modifyed: Xudong Hu         on Mar 24, 1999
*  Modified: Stephen Nestinger on Aug 21, 2006
*************************************************/

#ifndef Robot1_h
#define Robot1_h

#define ROBOT1 1
#define ROBOT1_PMAC 0          // PMAC device number robot is attached to
#define ROBOT1_NUM_JOINTS  6

#define LEFT   0b000
#define RIGHT  0b001
#define ABOVE  0b000
#define BELOW  0b010
#define FLIP   0b100
#define NOFLIP 0b000


/* DAC Safety Values */

long robot1_dac_safe[] = 
{
   5000,  /* DAC output full range is -32767 to +32767 (16 bits for +/-10V) */
   5000,
   5000,
   5000,
   5000,
   5000
};
 
/* Calibration Data :
     Calibrate potentiometer value using formula:
        y[i] = slope[i] * x[i] + yint[i]
     where x[i] is in Volts and y[i] is the required number of rev to
     get to the ready position. y[i] is then multiplied by the number
     of encoder counts per rev to get the encoder counts between the
     current joint position and its ready position.

     Ready position found to be at:
             VOLTS    PMAC 2 TURBO UNIPOLAR 12BIT VALUE
            ------    ---------------------------------
       x[1]=2.5104    2056
       x[2]=2.5177    2060
       x[3]=2.5299    2071
       x[4]=2.6948    2206
       x[5]=2.5336    2076
       x[6]=2.3944    1961
*/

double slope[] =               
{
    22.37756157,
    22.46972275,
    22.56998253,
    22.44226074,
    22.43869209,
    22.33512497
};

double yint[] =  
{
    -56.1766306,
    -56.5720210,
    -57.0997988,
    -60.4774042,
    -56.8506703,
    -53.4792232
};


double robot1_counts_rev[] =
{ 
   1000,                /* encoder 1000 counts/revolution for joint 1 */
   800,                 /* encoder 800 counts/revolution for joint 2 */
   1000,                /* encoder 1000 counts/revolution for joint 3 */
   1000,                /* encoder 1000 counts/revolution for joint 4 */
   1000,                /* encoder 1000 counts/revolution for joint 5 */
   1000                 /* encoder 1000 counts/revolution for joint 6 */
};
 
double robot1_enc_offset[] = 
{
   0,
   21563,               /* joint2 offset 90 degrees */
   13427,               /* joint3 offset 90 degrees */
   0,
   0,
   0
};


/*  PUMA DATA  

    Gear and Cross Coupling Parameters  

#define   G1        -62.6111    joint 1 gear ratio 
#define   G2        107.815     joint 2 gear ratio 
#define   G3        -53.7063    joint 3 gear ratio
#define   G4        76.03636    joint 4 gear ratio 
#define   G5        71.923      joint 5 gear ratio 
#define   G6        76.686      joint 6 gear ratio 
#define   G45       -0.013904   joint 4 and 5 cross coupling parameter 
#define   G46       -0.013040   joint 4 and 5 cross coupling parameter 
#define   G56       -0.180556   joint 4 and 5 cross coupling parameter 


               ---                                                  ---
               |                                                      |
               |     1/G1     0     0          0          0     0     |
               |                                                      |
               |      0     1/G2    0          0          0     0     |
               |                                                      |
               |      0       0   1/G3         0          0     0     |
Theta[j] =     |                                                      |Theta[m]
               |      0       0     0        1/G4         0     0     |
               |                                                      |
               |      0       0     0       G45/G4      1/G5    0     |
               |                                                      |
               |      0       0     0 (G46+G56+G45)/G4 G56/G5  1/G6   |
               |                                                      |
               ---                                                  ___
*/

array double robot1_JM_matrix[][6] = 
{{   -0.015972, 0.0,      0.0,       0.0,       0.0,       0.0      },
 {   0.0,       0.009275, 0.0,       0.0,       0.0,       0.0      },
 {   0.0,       0.0,      -0.018620, 0.0,       0.0,       0.0      },
 {   0.0,       0.0,      0.0,       0.013152,  0.0,       0.0      },
 {   0.0,       0.0,      0.0,       -0.000183, 0.013904,  0.0      },
 {   0.0,       0.0,      0.0,       -0.000171, -0.002510, 0.013040 }
};
/* define robotic parament:by Xudong Hu
   robot1ZeroPosition: Transformation matrix for zero position 
   robot1_B: TF matrix from base to coordinate
   robot1_H: TF matrix from link6 to the tool coordinate frame
   robot1_alpha: DH parament
   robot1_d: DH parament
   robot1_a: DH parament  */

array double robot1ZeroPosition[4][4]={  {1.0, 0.0, 0.0, 411.48},
              	                         {0.0, 1.0, 0.0, 149.09},
		                                 {0.0, 0.0, 1.0, 433.07},
		                                 {0.0, 0.0, 0.0, 1.0   }};

/* base matrix */
array double robot1_B[4][4] = {{1, 0, 0, 0},
                               {0, 1, 0, 0},
                               {0, 0, 1, 0},
                               {0, 0, 0, 1}};

/* hand matrix */
array double robot1_H[4][4] = {{1, 0, 0, 0    },
                               {0, 1, 0, 0    },
                               {0, 0, 1, 165.5},
                               {0, 0, 0, 1    }};

array double robot1_alpha[6] = {-90, 0,      90,    -90,    90, 0},
                 robot1_d[6] = {0,   149.09, 0,     433.07, 0,  0},
                 robot1_a[6] = {0,   431.8, -20.32, 0,      0,  0};

  /*  Inertia Parameters, not used in current version */

#define   Jm1            198e-6     /* Units = kg-m^2 */
#define   Jm2            203e-6
#define   Jm3            202e-6
#define   Jm4            18.3e-6
#define   Jm5            18.3e-6
#define   Jm6            18.3e-6


//-------------------------------------------------------------------------
// The following was added for inverse dynamics computation 
// by Yu-Cheng Chou 6/13/2007 

// link mass (kg)
array double m[6] = {0, 17.4, 4.8, 0.82, 0.34, 0.09};

// position of the c.o.m. of link i w.r.t. frame i (m)
array double s[6][3] = {0, 0, 0,
                        0.068, 0.006, -0.016,
                        0, -0.07, 0.014,
                        0, 0, -0.019,
                        0, 0, 0,
                        0, 0, 0.032};

// Moments of inertia about the c.o.m. of link i (kg-m^2)
array double I[6][3][3] = {0, 0, 0, 0, 0, 0, 0, 0, 0.35,
                           0.13, 0, 0, 0, 0.524, 0, 0, 0, 0.539,
                           0.066, 0, 0, 0, 0.0125, 0, 0, 0, 0.086,
                           0.0018, 0, 0, 0, 0.0018, 0, 0, 0, 0.0013,
                           0.0003, 0, 0, 0, 0.0003, 0, 0, 0, 0.0004,
                           0.00015, 0, 0, 0, 0.00015, 0, 0, 0, 0.00004};

// direction vector z0
array double z[3] = {0, 0, 1};

// initial condition w_0 
array double w_0[3] = {0, 0, 0};

// initial condition w_0_dot
array double w_0_dot[3] = {0, 0, 0};

// initial condition v_0_dot (m/s^2)
array double v_0_dot[3] = {0, 0, -9.8062};
//-------------------------------------------------------------------------

/* PMAC Data */

char* robot1_init[][69] = 

{{ /* Motor 1 */
    /* Definition */
    "i0100 = 1",         /* Motor Enable                               */
    "i0101 = 0",         /* Commutation Disable                        */
    "i0102 = $78002",    /* DAC Output Address                         */
    "i0103 = $3501",     /* Pos FB Data Address                        */
    "i0104 = $3501",     /* Vel FB Data Address                        */
    "i0105 = $35C0",     /* Master Position Address                    */
    "i0106 = 0",         /* Master Follow Enable                       */
    "i0107 = 96",        /* Master Scale Factor                        */
    "i0108 = 96",        /* Position Scale Factory                     */
    "i0109 = 96",        /* Velocity Scale Factor                      */
    "i0110 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0111 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0112 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0113 =  26088",    /* Software Pos Limit +, Unit: Encoder Counts */
    "i0114 = -26088",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0115 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0116 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0117 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0118 = 0.0",     /* Reserved                                   */
    "i0119 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0120 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0121 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0122 = 3",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0123 = 1",         /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0124 = $120001",   /* Motor Flag Mode Control                    */
    "i0125 = $78000",    /* Motor Flag Address                         */
    "i0126 = 0",         /* Motor Home Offset                          */
    "i0127 = 0",         /* Motor Position Rollover Range              */
    "i0128 = 200",       /* Motor In-Position Band                     */
    "i0129 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0130 = 800000",    /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0131 = 3500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0132 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0133 = 10000",     /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0134 = 1",         /* PID Inte Mode                              */
    "i0135 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0136 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0137 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0138 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0139 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0140 = 0",         /* Trajectory Filter Constant                 */
    "i0141 = 0",         /* Desired Position Limit Band                */

    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0155 = $0",        /* Commutation Table Address Offset           */
    "i0156 = 0",         /* Commutation Delay Compensation             */
    "i0157 = 0",         /* Continuous Current Limit                   */
    "i0158 = 0",         /* Integrated Current Limit                   */
    "i0159 = 0",         /* User-Written Servo/Phase Enable            */
    "i0160 = 0",         /* Servo Cycle Period Extension Period        */
    "i0161 = 0",         /* Current-Loop Integral Gain                 */
    "i0162 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0163 = 4194304",   /* Integration Limit                          */
    "i0164 = 0",         /* Deadband Gain Factor                       */
    "i0165 = 16",        /* Deadband Size                              */
    "i0166 = 6527",      /* PWM Scale Factor                           */
    "i0167 = 4194304",   /* Position Error Limit                       */
    "i0168 = 0",         /* Friction Feedforward                       */
    "i0169 = 9800",      /* Output Command Limit, 16 bits DAC, ~10V    */

    /* Backlash */
    "i0185 = 0",         /* Backlash take up rate                      */
    "i0186 = 0",         /* Backlash size                              */
    "i0187 = 64",        /* Backlash hysteresis                        */

    /* Servo IC Setup - Encoder Setup*/
    "i7010 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7011 = 0",         /* Position Compare Channel Select              */
    "i7012 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7013 = 0",         /* Capture Flag Select Control                  */
    "i7014 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7015 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7016 = 3",         /* Channel n Output Mode Select                 */
    "i7017 = 0",         /* Output Invert Control                        */
    "i7018 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 },
 { /* Motor 2 */
    /* Definition */
    "i0200 = 1",         /* Motor Enable                               */
    "i0201 = 0",         /* Commutation Disable                        */
    "i0202 = $7800A",    /* DAC Output Address                         */
    "i0203 = $3502",     /* Pos FB Data Address                        */
    "i0204 = $3502",     /* Vel FB Data Address                        */
    "i0205 = $35C0",     /* Master Position Address                    */
    "i0206 = 0",         /* Master Follow Enable                       */
    "i0207 = 96",        /* Master Scale Factor                        */
    "i0208 = 96",        /* Position Scale Factory                     */
    "i0209 = 96",        /* Velocity Scale Factor                      */
    "i0210 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0211 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0212 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0213 = 23000",     /* Software Pos Limit +, Unit: Encoder Counts */
    "i0214 = -23000",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0215 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0216 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0217 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0218 = 0.0",     /* Reserved                                   */
    "i0219 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0220 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0221 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0222 = 3",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0223 = 1",        /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0224 = $120001",   /* Motor Flag Mode Control                    */
    "i0225 = $78008",    /* Motor Flag Address                         */
    "i0226 = 0",         /* Motor Home Offset                          */
    "i0227 = 0",         /* Motor Position Rollover Range              */
    "i0228 = 200",       /* Motor In-Position Band                     */
    "i0229 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0230 = 800000",    /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0231 = 3500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0232 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0233 = 20000",       /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0234 = 1",         /* PID Inte Mode                              */
    "i0235 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0236 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0237 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0238 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0239 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0240 = 0",         /* Trajectory Filter Constant                 */
    "i0241 = 0",         /* Desired Position Limit Band                */


    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0255 = $0",        /* Commutation Table Address Offset           */
    "i0256 = 0",         /* Commutation Delay Compensation             */
    "i0257 = 0",         /* Continuous Current Limit                   */
    "i0258 = 0",         /* Integrated Current Limit                   */
    "i0259 = 0",         /* User-Written Servo/Phase Enable            */
    "i0260 = 0",         /* Servo Cycle Period Extension Period        */
    "i0261 = 0",         /* Current-Loop Integral Gain                 */
    "i0262 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0263 = 4194304",   /* Integration Limit                          */
    "i0264 = 0",         /* Deadband Gain Factor                       */
    "i0265 = 16",        /* Deadband Size                              */
    "i0266 = 6527",      /* PWM Scale Factor                           */
    "i0267 = 4194304",   /* Position Error Limit                       */
    "i0268 = 0",         /* Friction Feedforward                       */
    "i0269 = 15000",     /* Output Command Limit, 16 bits DAC, ~10V    */

    /* Backlash */
    "i0285 = 0",         /* */
    "i0286 = 0",         /* */
    "i0287 = 64",        /* */

    /* Servo IC Setup - Encoder Setup*/
    "i7020 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7021 = 0",         /* Position Compare Channel Select              */
    "i7022 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7023 = 0",         /* Capture Flag Select Control                  */
    "i7024 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7025 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7026 = 3",         /* Channel n Output Mode Select                 */
    "i7027 = 0",         /* Output Invert Control                        */
    "i7028 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 },
 { /* Motor 3 */
    /* Definition */
    "i0300 = 1",         /* Motor Enable                               */
    "i0301 = 0",         /* Commutation Disable                        */
    "i0302 = $78012",    /* DAC Output Address                         */
    "i0303 = $3503",     /* Pos FB Data Address                        */
    "i0304 = $3503",     /* Vel FB Data Address                        */
    "i0305 = $35C0",     /* Master Position Address                    */
    "i0306 = 0",         /* Master Follow Enable                       */
    "i0307 = 96",        /* Master Scale Factor                        */
    "i0308 = 96",        /* Position Scale Factory                     */
    "i0309 = 96",        /* Velocity Scale Factor                      */
    "i0310 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0311 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0312 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0313 = 19000",     /* Software Pos Limit +, Unit: Encoder Counts */
    "i0314 = -19000",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0315 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0316 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0317 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0318 = 0.0",     /* Reserved                                   */
    "i0319 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0320 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0321 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0322 = 3",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0323 = 1",        /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0324 = $120001",   /* Motor Flag Mode Control                    */
    "i0325 = $78010",    /* Motor Flag Address                         */
    "i0326 = 0",         /* Motor Home Offset                          */
    "i0327 = 0",         /* Motor Position Rollover Range              */
    "i0328 = 200",       /* Motor In-Position Band                     */
    "i0329 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0330 = 800000",    /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0331 = 3500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0332 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0333 = 20000",     /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0334 = 1",         /* PID Inte Mode                              */
    "i0335 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0336 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0337 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0338 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0339 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0340 = 0",         /* Trajectory Filter Constant                 */
    "i0341 = 0",         /* Desired Position Limit Band                */

    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0355 = $0",        /* Commutation Table Address Offset           */
    "i0356 = 0",         /* Commutation Delay Compensation             */
    "i0357 = 0",         /* Continuous Current Limit                   */
    "i0358 = 0",         /* Integrated Current Limit                   */
    "i0359 = 0",         /* User-Written Servo/Phase Enable            */
    "i0360 = 0",         /* Servo Cycle Period Extension Period        */
    "i0361 = 0",         /* Current-Loop Integral Gain                 */
    "i0362 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0363 = 4194304",   /* Integration Limit                          */
    "i0364 = 0",         /* Deadband Gain Factor                       */
    "i0365 = 16",        /* Deadband Size                              */
    "i0366 = 6527",      /* PWM Scale Factor                           */
    "i0367 = 4194304",   /* Position Error Limit                       */
    "i0368 = 0",         /* Friction Feedforward                       */
    "i0369 = 9800",      /* Output Command Limit, 16 bits DAC, ~10V    */

    /* Backlash */
    "i0385 = 0",         /* */
    "i0386 = 0",         /* */
    "i0387 = 64",        /* */

    /* Servo IC Setup - Encoder Setup*/
    "i7030 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7031 = 0",         /* Position Compare Channel Select              */
    "i7032 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7033 = 0",         /* Capture Flag Select Control                  */
    "i7034 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7035 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7036 = 3",         /* Channel n Output Mode Select                 */
    "i7037 = 0",         /* Output Invert Control                        */
    "i7038 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 },
 { /* Motor 4 */
    /* Definition */
    "i0400 = 1",         /* Motor Enable                               */
    "i0401 = 0",         /* Commutation Disable                        */
    "i0402 = $7801A",    /* DAC Output Address                         */
    "i0403 = $3504",     /* Pos FB Data Address                        */
    "i0404 = $3504",     /* Vel FB Data Address                        */
    "i0405 = $35C0",     /* Master Position Address                    */
    "i0406 = 0",         /* Master Follow Enable                       */
    "i0407 = 96",        /* Master Scale Factor                        */
    "i0408 = 96",        /* Position Scale Factory                     */
    "i0409 = 96",        /* Velocity Scale Factor                      */
    "i0410 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0411 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0412 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0413 = 19000",     /* Software Pos Limit +, Unit: Encoder Counts */
    "i0414 = -19000",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0415 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0416 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0417 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0418 = 0.0",     /* Reserved                                   */
    "i0419 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0420 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0421 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0422 = 3",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0423 = 1",        /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0424 = $120001",   /* Motor Flag Mode Control                    */
    "i0425 = $78018",    /* Motor Flag Address                         */
    "i0426 = 0",         /* Motor Home Offset                          */
    "i0427 = 0",         /* Motor Position Rollover Range              */
    "i0428 = 200",       /* Motor In-Position Band                     */
    "i0429 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0430 = 1000000",   /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0431 = 7500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0432 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0433 = 10000",     /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0434 = 1",         /* PID Inte Mode                              */
    "i0435 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0436 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0437 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0438 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0439 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0440 = 0",         /* Trajectory Filter Constant                 */
    "i0441 = 0",         /* Desired Position Limit Band                */

    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0455 = $0",        /* Commutation Table Address Offset           */
    "i0456 = 0",         /* Commutation Delay Compensation             */
    "i0457 = 0",         /* Continuous Current Limit                   */
    "i0458 = 0",         /* Integrated Current Limit                   */
    "i0459 = 0",         /* User-Written Servo/Phase Enable            */
    "i0460 = 0",         /* Servo Cycle Period Extension Period        */
    "i0461 = 0",         /* Current-Loop Integral Gain                 */
    "i0462 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0463 = 4194304",   /* Integration Limit                          */
    "i0464 = 0",         /* Deadband Gain Factor                       */
    "i0465 = 16",        /* Deadband Size                              */
    "i0466 = 6527",      /* PWM Scale Factor                           */
    "i0467 = 4194304",   /* Position Error Limit                       */
    "i0468 = 0",         /* Friction Feedforward                       */
    "i0469 = 9800",      /* Output Command Limit, 16 bits DAC, ~10V    */

    /* Backlash */
    "i0485 = 0",         /* */
    "i0486 = 0",         /* */
    "i0487 = 64",        /* */

    /* Servo IC Setup - Encoder Setup*/
    "i7040 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7041 = 0",         /* Position Compare Channel Select              */
    "i7042 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7043 = 0",         /* Capture Flag Select Control                  */
    "i7044 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7045 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7046 = 3",         /* Channel n Output Mode Select                 */
    "i7047 = 0",         /* Output Invert Control                        */
    "i7048 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 },
 { /* Motor 5 */
    /* Definition */
    "i0500 = 1",         /* Motor Enable                               */
    "i0501 = 0",         /* Commutation Disable                        */
    "i0502 = $78102",    /* DAC Output Address                         */
    "i0503 = $3505",     /* Pos FB Data Address                        */
    "i0504 = $3505",     /* Vel FB Data Address                        */
    "i0505 = $35C0",     /* Master Position Address                    */
    "i0506 = 0",         /* Master Follow Enable                       */
    "i0507 = 96",        /* Master Scale Factor                        */
    "i0508 = 96",        /* Position Scale Factory                     */
    "i0509 = 96",        /* Velocity Scale Factor                      */
    "i0510 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0511 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0512 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0513 = 18000",     /* Software Pos Limit +, Unit: Encoder Counts */
    "i0514 = -18000",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0515 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0516 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0517 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0418 = 0.0",     /* Reserved                                   */
    "i0519 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0520 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0521 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0522 = 3",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0523 = 1",        /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0524 = $120001",   /* Motor Flag Mode Control                    */
    "i0525 = $78100",    /* Motor Flag Address                         */
    "i0526 = 0",         /* Motor Home Offset                          */
    "i0527 = 0",         /* Motor Position Rollover Range              */
    "i0528 = 200",       /* Motor In-Position Band                     */
    "i0529 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0530 = 1000000",   /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0531 = 7500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0532 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0533 = 10000",     /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0534 = 1",         /* PID Inte Mode                              */
    "i0535 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0536 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0537 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0538 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0539 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0540 = 0",         /* Trajectory Filter Constant                 */
    "i0541 = 0",         /* Desired Position Limit Band                */

    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0555 = $0",        /* Commutation Table Address Offset           */
    "i0556 = 0",         /* Commutation Delay Compensation             */
    "i0557 = 0",         /* Continuous Current Limit                   */
    "i0558 = 0",         /* Integrated Current Limit                   */
    "i0559 = 0",         /* User-Written Servo/Phase Enable            */
    "i0560 = 0",         /* Servo Cycle Period Extension Period        */
    "i0561 = 0",         /* Current-Loop Integral Gain                 */
    "i0562 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0563 = 4194304",   /* Integration Limit                          */
    "i0564 = 0",         /* Deadband Gain Factor                       */
    "i0565 = 16",        /* Deadband Size                              */
    "i0566 = 6527",      /* PWM Scale Factor                           */
    "i0567 = 4194304",   /* Position Error Limit                       */
    "i0568 = 0",         /* Friction Feedforward                       */
    "i0569 = 9800",      /* Output Command Limit, 16 bits DAC, ~10V    */


    /* Backlash */
    "i0585 = 0",         /* */
    "i0586 = 0",         /* */
    "i0587 = 64",        /* */

    /* Servo IC Setup - Encoder Setup*/
    "i7110 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7111 = 0",         /* Position Compare Channel Select              */
    "i7112 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7113 = 0",         /* Capture Flag Select Control                  */
    "i7114 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7115 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7116 = 3",         /* Channel n Output Mode Select                 */
    "i7117 = 0",         /* Output Invert Control                        */
    "i7118 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 },
 { /* Motor 6 */
    /* Definition */
    "i0600 = 1",         /* Motor Enable                               */
    "i0601 = 0",         /* Commutation Disable                        */
    "i0602 = $7810A",    /* DAC Output Address                         */
    "i0603 = $3506",     /* Pos FB Data Address                        */
    "i0604 = $3506",     /* Vel FB Data Address                        */
    "i0605 = $35C0",     /* Master Position Address                    */
    "i0606 = 0",         /* Master Follow Enable                       */
    "i0607 = 96",        /* Master Scale Factor                        */
    "i0608 = 96",        /* Position Scale Factory                     */
    "i0609 = 96",        /* Velocity Scale Factor                      */
    "i0610 = $0",        /* Power-on Servo Position Address            */

    /* Safety */
    "i0611 = 0",         /* Disable  Motor Fatal following Error limit */
    "i0612 = 0",         /* Disable  Motor Warn following Error limit  */
    "i0613 = 32768",     /* Software Pos Limit +, Unit: Encoder Counts */
    "i0614 = -32768",    /* Software Pos Limit -, Unit: Encoder Counts */
    "i0615 = 0.25",      /* Pos Limit Decel Rate, Unit: Counts/msec^2  */
    "i0616 = 0.2",       /* Max Velocity, Unit: Counts/msec^2          */
    "i0617 = 0.02",      /* Max Acceleration, Unit: Counts/msec^2      */
    //"i0418 = 0.0",     /* Reserved                                   */
    "i0619 = 0.20",      /* Max Jog Acceleration, Unit: Counts/msec^2  */

    /* Movement */
    "i0620 = 1",         /* Motor Jog/Home Accel Time, Unit: msec      */
    "i0621 = 0",         /* Motor Jog/Home S-Curve Time,   Unit: msec  */
    "i0622 = 4",         /* Motor Jog Speed, Unit: Counts/msec         */
    "i0623 = 1",        /* Motor Home Speed & Dir, Unit: Counts/msec  */
    "i0624 = $120001",   /* Motor Flag Mode Control                    */
    "i0625 = $78108",    /* Motor Flag Address                         */
    "i0626 = 0",         /* Motor Home Offset                          */
    "i0627 = 0",         /* Motor Position Rollover Range              */
    "i0628 = 200",       /* Motor In-Position Band                     */
    "i0629 = 0",         /* Motor Output/1st Phase Offset              */

    /* Basic Servo */
    "i0630 = 1000000",   /* PID Proportional Gain, Unit: DAVbits/Count */
    "i0631 = 7500",      /* PID Derivative Gain, DACbits/(Counts/cycle)*/
    "i0632 = 1500",      /* PID Vel FF Gain, DACbits/(Counts/cycle)    */
    "i0633 = 10000",     /* PID Inte Gain, DACbits/(Counts*cycles)     */
    "i0634 = 1",         /* PID Inte Mode                              */
    "i0635 = 0",         /* PID Accel FF Gain, DACbits/(Counts/cycle^2)*/
    "i0636 = 0",         /* PID Notch Filter Coefficient N1            */
    "i0637 = 0",         /* PID Notch Filter Coefficient N2            */
    "i0638 = 0",         /* PID Notch Filter Coefficient D1            */
    "i0639 = 0",         /* PID Notch Filter Coefficient D2            */
    "i0640 = 0",         /* Trajectory Filter Constant                 */
    "i0641 = 0",         /* Desired Position Limit Band                */

    /* Servo/Commutation Modifiers */
    /*   Motors we use are self commutating */
    "i0655 = $0",        /* Commutation Table Address Offset           */
    "i0656 = 0",         /* Commutation Delay Compensation             */
    "i0657 = 0",         /* Continuous Current Limit                   */
    "i0658 = 0",         /* Integrated Current Limit                   */
    "i0659 = 0",         /* User-Written Servo/Phase Enable            */
    "i0660 = 0",         /* Servo Cycle Period Extension Period        */
    "i0661 = 0",         /* Current-Loop Integral Gain                 */
    "i0662 = 0",         /* Current-Loop Forward-Path Proportional Gain*/
    "i0663 = 4194304",   /* Integration Limit                          */
    "i0664 = 0",         /* Deadband Gain Factor                       */
    "i0665 = 16",        /* Deadband Size                              */
    "i0666 = 6527",      /* PWM Scale Factor                           */
    "i0667 = 4194304",   /* Position Error Limit                       */
    "i0668 = 0",         /* Friction Feedforward                       */
    "i0669 = 9800",      /* Output Command Limit, 16 bits DAC, ~10V    */

    /* Backlash */
    "i0685 = 0",         /* */
    "i0686 = 0",         /* */
    "i0687 = 64",        /* */

    /* Servo IC Setup - Encoder Setup*/
    "i7120 = 7",         /* Channel Encoder/Timer Decode Control         */
                         /*   Set to Encoder Dir Sense, X4 quadrature CW */
    "i7121 = 0",         /* Position Compare Channel Select              */
    "i7122 = 1",         /* Channel Capture Control, Rising edge of CHCn */
    "i7123 = 0",         /* Capture Flag Select Control                  */
    "i7124 = 0",         /* Channel n Encoder Gated Index Select         */
    "i7125 = 0",         /* Encoder Index Gate State/Demux Control       */
    "i7126 = 3",         /* Channel n Output Mode Select                 */
    "i7127 = 0",         /* Output Invert Control                        */
    "i7128 = 0",         /* PFM Direction Signal Invert Control          */
    NULL
 }
}; 

char* robot1_kill_motors[] = 
{
   "#1K",                     /* Kill Motor 1 output */
   "#2K",                     /* Kill Motor 2 output */
   "#3K",                     /* Kill Motor 3 output */
   "#4K",                     /* Kill Motor 4 output */
   "#5K",                     /* Kill Motor 5 output */
   "#6K",                     /* Kill Motor 6 output */
   NULL
};

char* robot1_open_loop[] = 
{
    "#1o0",                   /* Opening Loop for Jt 1     */
    "#2o0",                   /* Opening Loop for Jt 2     */
    "#3o0",                   /* Opening Loop for Jt 3     */
    "#4o0",                   /* Opening Loop for Jt 4     */
    "#5o0",                   /* Opening Loop for Jt 5     */
    "#6o0",                   /* Opening Loop for Jt 6     */
    NULL     
};

char* robot1_disable_motor[] =
{
    "i0100 = 0",               /* Disable Motor 1           */
    "i0200 = 0",               /* Disable Motor 2           */
    "i0300 = 0",               /* Disable Motor 3           */
    "i0400 = 0",               /* Disable Motor 4           */
    "i0500 = 0",               /* Disable Motor 5           */
    "i0600 = 0",               /* Disable Motor 6           */
    NULL
};

char* robot1_home[] =
{
   "0",                 /* Joint 1 Home position       */
   "0",                 /* Joint 2 Home position       */
   "180",               /* Joint 3 Home position       */
   "0",                 /* Joint 4 Home position       */
   "0",                 /* Joint 5 Home position       */
   "0",                 /* Joint 6 Home position       */
    NULL
}; 

char* robot1_homing[] =
{
   "#1hm",                 /* Homing Motor 1            */
   "#2hm",                 /* Homing Motor 2            */
   "#3hm",                 /* Homing Motor 3            */
   "#4hm",                 /* Homing Motor 4            */
   "#5hm",                 /* Homing Motor 5            */
   "#6hm",                 /* Homing Motor 6            */
    NULL
}; 


char* robot1_enc_zero [] =
{
   "homez#1",           /* Reset encoder 1         */
   "homez#2",           /* Reset encoder 2         */
   "homez#3",           /* Reset encoder 3         */
   "homez#4",           /* Reset encoder 4         */
   "homez#5",           /* Reset encoder 5         */
   "homez#6",           /* Reset encoder 6         */
   NULL
};


char* robot1_ready[] = 
{
   "0",                 /*  Joint1 ready position   */
   "-90",               /*  Joint2 ready position   */
   "90",                /*  Joint3 ready position   */
   "0",                 /*  Joint4 ready position   */
   "0",                 /*  Joint5 ready position   */
   "0",                 /*  Joint6 ready position   */
   NULL
};

char* robot1_zero[] = 
{
   "0",                 /*  Joint 1 zero position   */
   "0",                 /*  Joint 2 zero position   */
   "0",                 /*  Joint 3 zero position   */
   "0",                 /*  Joint 4 zero position   */
   "0",                 /*  Joint 5 zero position   */
   "0",                 /*  Joint 6 zero position   */
   NULL
};

 

char* robot1_loop_closure[] = 
{
    "#1j/",             /* Restore joint 1 closed-loop control */
    "#2j/",             /* Restore joint 2 closed-loop control */
    "#3j/",             /* Restore joint 3 closed-loop control */
    "#4j/",             /* Restore joint 4 closed-loop control */
    "#5j/",             /* Restore joint 5 closed-loop control */
    "#6j/",             /* Restore joint 6 closed-loop control */
    NULL
};

char* robot1_pot_address[] = 
{
    "m5061->Y:$3400,12,12,U",    /* ADC 1 Address Joint 1     */
    "m5062->Y:$3402,12,12,U",    /* ADC 2 Address Joint 2     */
    "m5063->Y:$3404,12,12,U",    /* ADC 3 Address Joint 3     */
    "m5064->Y:$3406,12,12,U",    /* ADC 4 Address Joint 4     */
    "m5065->Y:$3408,12,12,U",    /* ADC 5 Address Joint 5     */
    "m5066->Y:$340A,12,12,U",    /* ADC 6 Address Joint 6     */
    NULL
};

char * robot1_grip_enable[] = 
{
    "m32 = 255",                /* 255 == 0b11111111 for 8 digital I/O as input */
    "m33 = 1",                  
    "m0  = 0",                  /* start with gripper  closed */
    NULL
};

char * robot1_grip_close[] = 
{
    "m0 = 0",                  /* close gripper               */
    NULL
};

char * robot1_grip_open[] = 
{
    "m0 = 1",                  /* open gripper               */
    NULL
};

#endif                        /* Robot1_h                  */
